//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "SharesTableViewController.h"
#import "CustomTableViewCell.h"
#import "UIViewController+Dismiss.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+Share.h"
#import "UIViewController+Spinner.h"


@interface BaseTableViewController (Private)

@property (nonatomic, strong) NSString *emptyMessage;

@end

@interface SharesTableViewController () <UITableViewDataSource, CustomTableViewCellDelegate>
// stores list of PCShareItem object.
@property (nonatomic, strong) NSMutableArray *shares;
// stores next curser link.
@property (nonatomic, strong) NSString *nextLink;
// stores share total count.
@property (nonatomic, assign) NSInteger totalCount;


@end

@implementation SharesTableViewController

- (void)viewDidLoad {
    
    [super viewDidLoad];
    
    // Assigning empty message when no share to display.
    self.emptyMessage = NSLocalizedString(@"There is no share to display", nil);
    self.nextLink = nil;
    self.totalCount = 0;
    self.shares = [NSMutableArray new];
    [self reloadData]; // Reload the data to table view.
    
}

- (void)reloadData {
    
    [self showSpinner]; // start the loading spinner.
    /*
     @Description -
     This API is used to fetch list of user created share. Once the fetch process is done/failed result will come with completion block.
     
     @Parameters -
     PCGetShareAPIParams : This class store all the parameters required to fetch user shares.
     * accessToken are required parameter to call this API. For providing access token to SDK user have to logged In. If user will not pass required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
     * cursor, count, since, until  and filter  are  optional parameter for this API.
     * More information about parameters you will find in PCGetShareAPIParams.h class.
     
     @CompletionHandler -
     This completion block gets called once share fetch process is done/failed. completion block contains below parameter.
     PCAPIQueryResponse : Stores all the server response and error for this API.
     PCShare : PCShare stores all the defination of a user shares.
     
     @To fetch users share, you must take the following steps: -
     * Include VerizonCloudAPI.framework header file in your class.
     #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCGetShareAPIParams class and provide the required parameter.
     * Initalize the PCAPISDK object and call the fetchShare:withCompletionHandler: API.
     * Get the result(Success or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    // Sample code for fetchShare:withCompletionHandler: API
    
    // Initailize the PCGetShareAPIParams object and assign the required parameter.
    PCGetShareAPIParams *getParams = [[PCGetShareAPIParams alloc] init];
    getParams.cursor = self.nextLink;
    
    __weak SharesTableViewController *blocksafeSelf = self;
    
    // Initailize the PCAPISDK class and call the fetchShare:withCompletionHandler: API call.
    [[PCAPISDK sharedInstance] fetchShare:getParams withCompletionHandler:^(PCShare *share, PCAPIQueryResponse *response) {
         [self hideSpinner];
        // check the urlResponse status code and do the operation.
        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodeOK: {
                // API call successfully fetched list of share.
                blocksafeSelf.totalCount = [share.count integerValue];
                [blocksafeSelf.shares addObjectsFromArray:share.shareList];
                
                // Assign next link to variable to do pagination.
                if ([share.links count]) {
                    for (PCLink *nextLink in share.links) {
                        if ([nextLink.rel isEqualToString:@"next"]) {
                            blocksafeSelf.nextLink = [[nextLink.link componentsSeparatedByString:@"="] objectAtIndex:1]; // parsing the next link
                            break;
                        }
                    }
                }
                // Refresh the table view with updated data.
                [blocksafeSelf refreshView];
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                // API call failed because Session is no longer valid.
                [self showUnAuthorizedAlert];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                // API call failed because server unavailable.
                [self showServerUnavailableAlert];
                break;
            default:
                // API call failed due to other network error.
                [self showOperationFailedAlert:NSLocalizedString(@"Share", nil)];
                break;
        }
        
    }];
}

- (void)refreshView {
    
    if (self.shares.count) {
        [self.tableView reloadData]; // reload the table view.
    } else {
        [self showOKAlertWithTitle:NSLocalizedString(@"Empty", nil)
                        andMessage:self.emptyMessage attributedString:nil okAction:nil];
    }
}

#pragma mark -
#pragma mark UITableView data source method
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.shares.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    CustomTableViewCell *cell = (CustomTableViewCell *)[tableView dequeueReusableCellWithIdentifier:@"shareCell" forIndexPath:indexPath];
    
    PCShareItem *shareItem = [self.shares objectAtIndex:indexPath.row];
    
    cell.textLabel.text = [NSString stringWithFormat:@"%@ (%@)",shareItem.name,shareItem.totalResourceCount];
    cell.delegate = self;
    
    return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        
        PCShareItem *shareItem = [self.shares objectAtIndex:indexPath.row];
        [self executeAction:deleteShareAction forShare:shareItem];
    }
}


#pragma mark -
#pragma mark UITableView delegate method
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
}

- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath {
    return UITableViewCellEditingStyleDelete;
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath {
    if ( indexPath.row == [self.shares count] - 1 ) {
        if (([self.shares count] != self.totalCount) && self.nextLink != nil ) {
            [self reloadData];
        }
    }
}

#pragma mark -
#pragma mark CustomTableViewCell delegate

- (void)tableViewCellDidLongPressed:(UITableViewCell *)cell {
    
    NSIndexPath *indexPath = [self.tableView indexPathForCell:cell];
    PCShareItem *selectedShare = [self.shares objectAtIndex:indexPath.row];
    [self showActionsForSelectedShare:selectedShare];
}

#pragma mark -
#pragma mark Share Action callback

- (void)didBeginDoingShareAction:(ShareAction)action {
    
    [self showSpinner]; // start the loading spinner.
}

- (void)didFinishDoingAction:(ShareAction)action onShare:(id)share result:(id)result response:(id)response {
    
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    [self hideSpinner]; // stop the loading spinner.
    
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeNoContent: {
            
            switch (action) {
                case deleteShareAction: {
                        // Delete the share from the list and reload the table view.
                        [self.shares removeAllObjects];
                        self.nextLink = nil;
                        self.totalCount = 0;
                        [self reloadData];
                }
                    break;
                default:
                    break;
            }
        }
            break;
        case PCAPIResponseCodeUnauthorized:
             // API call failed because Session is no longer valid.
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            // API call failed because server unavailable.
            [self showServerUnavailableAlert];
            break;
        default:
            // API call failed due to other network error.
             [self showOperationFailedAlert:NSLocalizedString(@"Delete share", nil)];
            break;
    }
}


@end
